/***
 * OW2 FraSCAti Examples: Testing module
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Nicolas Dolet
 *
 * Contributor(s):
 */

package org.ow2.frascati.examples.test;

import org.junit.After;
import org.junit.Before;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.frascati.factory.Factory;
import org.ow2.frascati.factory.FactoryException;
import org.ow2.frascati.factory.FactoryInstantiationException;
import org.ow2.frascati.tinfi.TinfiDomain;

/**
 * Abstract class for testing SCA applications with FraSCAti
 * 
 */
public abstract class FraSCAtiTestCase {

  /**
   * The SCA composite to test
   */
  protected Component scaComposite;

  /**
   * Load a composite
   * 
   * @param name
   *          the composite name
   * @throws FactoryException, FactoryInstantiationException 
   */
  @Before
  public void loadComposite() throws FactoryException, FactoryInstantiationException {
    String compositeName = getComposite();
    System.out.println("Loading " + compositeName + " composite...");
    scaComposite = new Factory().getComposite(compositeName);
  }

  /**
   * Get a service on the SCA composite
   * 
   * @param <T>
   *          the class type for the return
   * @param serviceName
   *          the name of the service
   * @return the service named <code>serviceName</code> or null
   */
  protected <T> T getService(Class<T> cl, String serviceName) {
    System.out.println("Getting service " + serviceName + "...");
    try {
      return TinfiDomain.getService(scaComposite, cl, serviceName);
    } catch (NoSuchInterfaceException e) {
      System.err.println("No such service: " + serviceName);
      return null;
    }
  }

  /**
   * Close the SCA domain
   * 
   * @throws IllegalLifeCycleException
   *           if the domain cannot be closed
   * @throws NoSuchInterfaceException
   *           if the lifecycle controller of the component is not found
   */
  @After
  public void close() throws IllegalLifeCycleException,
      NoSuchInterfaceException {
    TinfiDomain.close(scaComposite);
  }

  // ---------------------------------------------------------------------------
  // Abstract methods
  // ---------------------------------------------------------------------------
  /**
   * Return the name of the composite to test
   * 
   * @return name of the composite to test
   */
  public abstract String getComposite();

}
