/***
 * OW2 FraSCAti Examples: Fscript reconfiguration
 * Copyright (C) 2010 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s): 
 * 
 */

package org.ow2.frascati.examples.reconfig.converter.explorer;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.NumberFormat;

import javax.swing.JLabel;
import javax.swing.JTextField;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.LayoutStyle;
import org.ow2.frascati.examples.reconfig.converter.CurrencyConverter;
import org.ow2.frascati.explorer.gui.AbstractSelectionPanel;

/**
 * Is the FraSCAti Explorer plugin to show {@link CurrencyConverter} instances.
 *
 * @author Christophe Demarey
 */
@SuppressWarnings("serial")
public class ConverterPanel extends AbstractSelectionPanel<CurrencyConverter> {
  private NumberFormat nf; // used to format double display
  private JLabel jLabelDollar;
  private JLabel jLabelEuro;
  private JTextField jTextFieldDollar;
  private JTextField jTextFieldEuro;

  /**
   * Default constructor creates the panel.
   */
  public ConverterPanel() {
    super();
    
    nf = NumberFormat.getInstance();
    nf.setMaximumFractionDigits(4);

    jLabelEuro = new JLabel();
    jLabelDollar = new JLabel();
    jTextFieldEuro = new JTextField();
    jTextFieldDollar = new JTextField();

    jLabelEuro.setText("Euro");
    jLabelEuro.setName("jLabelEuro");
    jLabelDollar.setText("Dollar");
    jLabelDollar.setName("jLabelDollar");
    jTextFieldEuro.setName("jTextFieldEuro");
    jTextFieldDollar.setName("jTextFieldDollar");

    // layout
    GroupLayout mainPanelLayout = new GroupLayout(this);
    this.setLayout(mainPanelLayout);
    mainPanelLayout.setHorizontalGroup(
        mainPanelLayout.createParallelGroup(GroupLayout.LEADING)
        .add(mainPanelLayout.createSequentialGroup()
            .addContainerGap()
            .add(mainPanelLayout.createParallelGroup(GroupLayout.LEADING)
                .add(mainPanelLayout.createSequentialGroup()
                    .add(mainPanelLayout.createParallelGroup(GroupLayout.CENTER)
                        .add(jLabelEuro)
                        .add(jTextFieldEuro, GroupLayout.PREFERRED_SIZE, 99, GroupLayout.PREFERRED_SIZE))
                    .add(18, 18, 18)
                    .add(mainPanelLayout.createParallelGroup(GroupLayout.CENTER)
                        .add(jLabelDollar)
                        .add(jTextFieldDollar, GroupLayout.PREFERRED_SIZE, 99, GroupLayout.PREFERRED_SIZE))))
            .addContainerGap(607, Short.MAX_VALUE))
    );
    mainPanelLayout.setVerticalGroup(
        mainPanelLayout.createParallelGroup(GroupLayout.LEADING)
        .add(mainPanelLayout.createSequentialGroup()
            .add(20, 20, 20)
            .add(mainPanelLayout.createParallelGroup(GroupLayout.TRAILING)
                .add(mainPanelLayout.createSequentialGroup()
                    .add(jLabelDollar)
                    .addPreferredGap(LayoutStyle.RELATED)
                    .add(jTextFieldDollar, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
                .add(mainPanelLayout.createSequentialGroup()
                    .add(jLabelEuro)
                    .addPreferredGap(LayoutStyle.RELATED)
                    .add(jTextFieldEuro, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)))
            .addPreferredGap(LayoutStyle.RELATED)
            .addContainerGap(545, Short.MAX_VALUE))
    );
    
    // actions
    jTextFieldEuro.addKeyListener( new KeyListener() {
        public void keyPressed(KeyEvent e) { }

        public void keyTyped(KeyEvent e) { }

        public void keyReleased(KeyEvent e) {
            if ( jTextFieldEuro.getText().length() > 0 ) {
                double value = Double.parseDouble( jTextFieldEuro.getText().replaceAll(",", ".") );
                double result = selected.euroToDollar(value);
                jTextFieldDollar.setText( nf.format(result) );
            }
        }
      }
    );
    jTextFieldDollar.addKeyListener( new KeyListener() {
        public void keyPressed(KeyEvent e) { }

        public void keyTyped(KeyEvent e) { }

        public void keyReleased(KeyEvent e) {
            if ( jTextFieldDollar.getText().length() > 0 ) {
                double value = Double.parseDouble( jTextFieldDollar.getText().replaceAll(",", ".") );
                double result = selected.dollarToEuro(value);
                jTextFieldEuro.setText( nf.format(result) );
            }
        }
      }
    );
    jTextFieldEuro.addFocusListener( new FocusListener() {
        public void focusLost(FocusEvent e) { }
        
        public void focusGained(FocusEvent e) {
            jTextFieldEuro.setText("");
        }
      }
    );
    jTextFieldDollar.addFocusListener( new FocusListener() {
        public void focusLost(FocusEvent e) { }
        
        public void focusGained(FocusEvent e) {
            jTextFieldDollar.setText("");
        }
      }
    );

  }
  
}
