/***
 * OW2 FraSCAti Examples : Fscript reconfiguration
 * Copyright (C) 2008-2010 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors: Christophe Demarey
 */
package org.ow2.frascati.examples.reconfig.converter.fscript;

import java.util.Set;
import java.util.TreeSet;

import org.objectweb.fractal.fscript.FScriptException;
import org.osoa.sca.annotations.Property;
import org.ow2.frascati.fscript.FraSCAtiScript;

/**
 * A SCA component used to dynamically reconfigure another SCA component: the
 * currency converter.
 * 
 * The FScript plugin <b>MUST</b> be activated!
 * 
 * @author Christophe Demarey.
 */
public class ExchangeRateUpdaterImpl implements ExchangeRateUpdater {
    @Property
    protected String reconfigurationScript;

    public void updateExchangeRate(double newValue) throws FScriptException {
        FraSCAtiScript fscript = FraSCAtiScript.getSingleton();
        
        fscript.execute("converter = $domain/scachild::reconfig/scachild::currency-converter;");
        fscript.execute("rate = $converter/scaproperty::exchangeRate;");
        fscript.execute("set-value($rate,"+newValue+")");
    }

    public void updateExchangeRateV2(double newValue) throws FScriptException {
        Set<String> loadedProc = new TreeSet<String>();
        FraSCAtiScript fscript = FraSCAtiScript.getSingleton();
        
        loadedProc = fscript.load(reconfigurationScript);
        for (String proc : loadedProc) {
            System.out.println("+-> Adding procedure '"+ proc +"' to FraSCAtiScript engine");
        }
        fscript.execute("reconfigure-converter("+newValue+")");
    }

}
